/////////////////////////////////////////////////////////////////////////////////////////
// Custom Super Color                                                     Version: 1.2 //
// "Want a Blue Super Sonic? It's yours."                               Author: Dark94 //
// Modify: ASK ME | Maintain: NO                              Just some good ol' fixes //
/////////////////////////////////////////////////////////////////////////////////////////
// Works with ANY color!                                                               //
//                                                 Vanilla, custom, and even animated! //
/////////////////////////////////////////////////////////////////////////////////////////
// COMMANDS:                                                                           //
// customsupercolor: -- -- -- -- -- -- -- -- -- -- -- -- -- -- Changes the Super color //
// supercolorteams: -- -- -- -- -- -- -- Forces Super Red/Blue for teams <ADMIN ONLY!> //
// csc_setup: -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- Get the Super Setup menu //
/////////////////////////////////////////////////////////////////////////////////////////

//////////////////////
// Initial thingies //
//////////////////////

local CSC_VERSION = "1.2"

local CSC_MenuTipDisappeared = false
local CSC_CustomColorFile = "client/CustomSuperColor/color.dat"
local CSC_TeamsColorsFile = "client/CustomSuperColor/teamcolorsforcing.dat"

local CSC_PrintIfNotSilent = function(player, text, silent)
	if silent
		return
	end

	CONS_Printf(player, text)
end

local CSC_GetSupercolorLastFrame = function(color) // Give a Super color, get its last frame.
	if not color
		return SKINCOLOR_SUPERGOLD5
	end

	color = $+4

	return color
end

local CSC_GiveColor = function(player, type) // Give us the type of color specified!
	if not type
		return SKINCOLOR_GREEN
	end

	local color = SKINCOLOR_GREEN

	if type == "current"
		color = player.skincolor
	end
	if type == "opposite"
		color = ColorOpposite(player.skincolor)
	end
	if type == "super"
		if player.mo
		and player.mo.valid
			color = CSC_GetSupercolorLastFrame(skins[player.mo.skin].supercolor)
		end
	end

	return color
end

local CSC_GiveSuperColor = function(player) // Give us our Super color!
	local anim_timer = leveltime // If we are NOT Super, animate the color with the level's time
	if player.powers[pw_super] // But... if we ARE Super, animate the color with our Super time
		anim_timer = player.powers[pw_super]
	end

	if not player.csc_color
	or (player.csc_color
	and player.csc_color == "none")
	and not (player.csc_teamcolors
	and gametyperules & GTR_TEAMS
	and (player.ctfteam == 1
	or player.ctfteam == 2))
		if player.mo
		and player.mo.valid
			local firstframe = CSC_GetSupercolorLastFrame(skins[player.mo.skin].supercolor)-4
			return firstframe+abs(((anim_timer>>1)%9)-4)
		else
			return SKINCOLOR_SUPERGOLD1+abs(((anim_timer>>1)%9)-4)
		end
	end

	for i = 0, 31
		if player == players[i]
			return SKINCOLOR_CUSTOM_SUPERPLAYER0+i
		end
	end
end

local CSC_ColorFlagToEscapeColor = function(color) // Give a color flag, get a escape color.
	local thefunnitable = { // This table will make the job easier
		[V_GRAYMAP] = "\x86", // Gray
		[V_INVERTMAP] = "\x8F", // Inverted/Black
		[V_BROWNMAP] = "\x8D", // Brown
		[V_REDMAP] = "\x85", // Red
		[V_ORANGEMAP] = "\x87", // Orange
		[V_YELLOWMAP] = "\x82", // Yellow
		[V_PERIDOTMAP] = "\x8B", // Lime/Peridot
		[V_GREENMAP] = "\x83", // Green
		[V_AQUAMAP] = "\x8A", // Aqua
		[V_SKYMAP] = "\x88", // Cyan/Sky
		[V_BLUEMAP] = "\x84", // Blue
		[V_AZUREMAP] = "\x8C", // Azure
		[V_PURPLEMAP] = "\x89", // Purple
		[V_MAGENTAMAP] = "\x81", // Magenta
		[V_ROSYMAP] = "\x8E" // Pink/Rosy
	}
	if color // Check 'color' isn't nil
	and thefunnitable[color] // Check 'color' can be converted
		return thefunnitable[color] // Return the converted color
	else
		return "\x80"
	end
end

local CSC_GivePreviousColor = function(color) // Get the color previous to your color
	if not color
		return SKINCOLOR_GREEN
	end

	local prevcolor = SKINCOLOR_GREEN

	if color == SKINCOLOR_WHITE
		prevcolor = #skincolors-1
	else
		prevcolor = color-1
	end
	while not skincolors[prevcolor].accessible do
		prevcolor = $-1
	end

	return prevcolor
end

local CSC_GiveNextColor = function(color) // Get the color next to your color
	if not color
		return SKINCOLOR_GREEN
	end

	local nextcolor = SKINCOLOR_GREEN
	local substractthingy = 1
	while not skincolors[#skincolors-substractthingy].accessible do
		substractthingy = $+1
	end

	if color == #skincolors-substractthingy
		nextcolor = SKINCOLOR_WHITE
	else
		nextcolor = color+1

		while not skincolors[nextcolor].accessible do
			nextcolor = $+1
		end
	end

	return nextcolor
end

local CSC_Borders = "CSC_SQ_BORDER"
local CSC_Corners = "CSC_SQ_CORNER"

// Box drawer recycled from other addon of mine
local CSC_DrawSquareBorders = function(v, x, y, xscale, yscale, flags, alignment)
	if not v
		return
	end
	if not x
		x = 0
	end
	if not y
		y = 0
	end
	if not xscale
	or (xscale and xscale < 6*FRACUNIT)
		xscale = 6*FRACUNIT
	end
	if not yscale
	or (yscale and yscale < 6*FRACUNIT)
		yscale = 6*FRACUNIT
	end
	local the_flags = nil
	if flags
		the_flags = flags
	end
	if alignment
		if alignment == "center"
		or alignment == "up-center"
			x = $-xscale/2
		end
		if alignment == "center-center"
			x = $-xscale/2
			y = $-yscale/2
		end
		if alignment == "down-center"
			x = $-xscale/2
			y = $-yscale
		end
		if alignment == "center-left"
			y = $-yscale/2
		end
		if alignment == "down-left"
			y = $-yscale
		end
		if alignment == "right"
		or alignment == "up-right"
			x = $-xscale
		end
		if alignment == "center-right"
			x = $-xscale
			y = $-yscale/2
		end
		if alignment == "down-right"
			x = $-xscale
			y = $-yscale
		end
	end

	x = $-v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT
	y = $-v.cachePatch(CSC_Corners.."_LU").height*FRACUNIT

	if xscale
	and yscale
		// Corners
		v.drawScaled(x, y, FRACUNIT, v.cachePatch(CSC_Corners.."_LU"), the_flags)
		v.drawScaled(x, y+yscale+(v.cachePatch(CSC_Corners.."_LU").width)*FRACUNIT, FRACUNIT, v.cachePatch(CSC_Corners.."_LD"), the_flags)
		v.drawScaled(x+xscale+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, y, FRACUNIT, v.cachePatch(CSC_Corners.."_RU"), the_flags)
		v.drawScaled(x+xscale+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, y+yscale+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, FRACUNIT, v.cachePatch(CSC_Corners.."_RD"), the_flags)

		// Borders
		v.drawStretched(x+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, y, xscale, FRACUNIT, v.cachePatch(CSC_Borders.."_UP"), the_flags) // Up
		v.drawStretched(x, y+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, FRACUNIT, yscale, v.cachePatch(CSC_Borders.."_LEFT"), the_flags) // Left
		v.drawStretched(x+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, y+yscale+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, xscale, FRACUNIT, v.cachePatch(CSC_Borders.."_DOWN"), the_flags) // Down
		v.drawStretched(x+xscale+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, y+v.cachePatch(CSC_Corners.."_LU").width*FRACUNIT, FRACUNIT, yscale, v.cachePatch(CSC_Borders.."_RIGHT"), the_flags) // Right
	end
end

/////////////////////////////////////////
// Input/Output (I/O for short) system //
/////////////////////////////////////////

// Save function for custom color
local CSC_SaveSuperColor = function(value)
	local thefile = io.openlocal(CSC_CustomColorFile, "w+")

	thefile:write(value)
	thefile:close()
end

// Load function for custom color
local CSC_LoadSuperColor = function(player, silent)
	local value2 = 0
	if silent
		value2 = 1
	end

	local thefile = io.openlocal(CSC_CustomColorFile)

	if thefile
		local thevalue = thefile:read("*a")

		if (tonumber(thevalue)
		and tonumber(thevalue) <= #skincolors
		and skincolors[thevalue])
		or R_GetColorByName(thevalue)
			COM_BufInsertText(player, 'customsupercolor "'..thevalue..'" '..value2)
		else
			COM_BufInsertText(player, "customsupercolor 0 "..value2)
			player.csc_setup_selectedcolor = SKINCOLOR_WHITE
		end

		thefile:close()
	else
		COM_BufInsertText(player, "customsupercolor 0 "..value2)
		player.csc_setup_selectedcolor = SKINCOLOR_WHITE
	end
end

// Command to change the color!
COM_AddCommand("customsupercolor", function(player, arg, silent)
	if not player
		print("\x85".."ERROR: ".."\x80".."You don't exist!") // Go to a map, please.
		return
	end

	if arg
		local og_arg = arg
		arg = string.lower(tostring(arg)) // Case-insensitive yey :D

		if (tonumber(arg) // Custom color
		and skincolors[tonumber(arg)]
		and not (tonumber(arg) >= SKINCOLOR_CUSTOM_SUPERPLAYER0
		and tonumber(arg) <= SKINCOLOR_CUSTOM_SUPERPLAYER0+31)
		and skincolors[tonumber(arg)].accessible)
		or (R_GetColorByName(arg)
		and not (R_GetColorByName(arg) >= SKINCOLOR_CUSTOM_SUPERPLAYER0
		and R_GetColorByName(arg) <= SKINCOLOR_CUSTOM_SUPERPLAYER0+31)
		and skincolors[R_GetColorByName(arg)].accessible)
			if tonumber(arg) // It's a number
				player.csc_color = tonumber(arg) // Change our Super color's value
				player.csc_setup_selectedcolor = tonumber(arg)
			else // It's a string
				player.csc_color = R_GetColorByName(arg) // Change our Super color's value
				player.csc_setup_selectedcolor = R_GetColorByName(arg)
			end

			CSC_SaveSuperColor(tonumber(player.csc_color)) // Save our preference
			CSC_PrintIfNotSilent(player, "Your new Super color is "..CSC_ColorFlagToEscapeColor(skincolors[player.csc_color].chatcolor)..skincolors[player.csc_color].name, silent) // Show us the new color!
		elseif arg == "opp" // Current opposite
		or arg == "opposite"
		or arg == "contrary"
		or arg == "inv"
		or arg == "invert"
		or arg == "oppcolor"
		or arg == "contrarycolor"
		or arg == "invcolor"
		or arg == "invertcolor"
			player.csc_color = "inv"
			CSC_SaveSuperColor("inv")
			CSC_PrintIfNotSilent(player, "Your new Super color is "..CSC_ColorFlagToEscapeColor(skincolors[ColorOpposite(player.skincolor)].chatcolor).."the opposite of your current color", silent) // Opposite color? Interesting...
		elseif arg == "true" // Current color
		or arg == "on"
		or arg == "color"
		or arg == "current"
		or arg == "currentcolor"
			player.csc_color = "current"
			CSC_SaveSuperColor("current")
			CSC_PrintIfNotSilent(player, "Your new Super color is "..CSC_ColorFlagToEscapeColor(skincolors[player.skincolor].chatcolor).."your current color", silent) // U just changed it to your own color!!!
		elseif arg == "0" // Default color
		or arg == "false"
		or arg == "off"
		or arg == "current"
		or arg == "skin"
		or arg == "default"
		or arg == "normal"
		or arg == "normalcolor"
		or arg == "pref"
		or arg == "prefsuper"
		or arg == "supercolor"
		or arg == "prefsupercolor"
			player.csc_color = "none"
			CSC_SaveSuperColor("0")
			CSC_PrintIfNotSilent(player, "Your new Super color is your default Super color", silent) // Changed back to default...
		else
			CSC_PrintIfNotSilent(player, "\x85".."ERROR: ".."\x82"..og_arg.."\x80".." isn't a valid color! Printing command guide...", silent) // ...really?
			CSC_PrintIfNotSilent(player, 'customsupercolor <[color]|"normal"|"current"|"opposite">', silent)
			CSC_PrintIfNotSilent(player, '"normal" sets your Super color to the default value', silent)
			CSC_PrintIfNotSilent(player, '"current" sets your Super color to the color you are using currently', silent)
			CSC_PrintIfNotSilent(player, '"opposite" sets your Super color to the opposite of your current color', silent)
		end
	else
		CONS_Printf(player, 'customsupercolor <[color]|"normal"|"current"|"opposite">') // Show a small guide if we don't get any argument
		CONS_Printf(player, '"normal" sets your Super color to the default value')
		CONS_Printf(player, '"current" sets your Super color to the color you are using currently')
		CONS_Printf(player, '"opposite" sets your Super color to the opposite of your current color')
		if player.csc_color == "none"
			CONS_Printf(player, "\x82".."Current value: ".."\x80".."Default")
		elseif player.csc_color == "current"
			CONS_Printf(player, "\x82".."Current value: "..CSC_ColorFlagToEscapeColor(skincolors[player.skincolor].chatcolor).."Current color")
		elseif player.csc_color == "inv"
			CONS_Printf(player, "\x82".."Current value: "..CSC_ColorFlagToEscapeColor(skincolors[ColorOpposite(player.skincolor)].chatcolor).."Current opposite")
		else
			CONS_Printf(player, "\x82".."Current value: "..CSC_ColorFlagToEscapeColor(skincolors[R_GetColorByName(player.csc_color)].chatcolor)..skincolors[R_GetColorByName(player.csc_color)].name)
		end
	end
end)

// Save function for team color forcing
local CSC_SaveSuperTeams = function(value)
	local thefile = io.openlocal(CSC_TeamsColorsFile, "w+")

	thefile:write(value)
	thefile:close()
end

// Load function for team color forcing
local CSC_LoadSuperTeams = function(player, silent)
	local value2 = 0
	if silent
		value2 = 1
	end

	local thefile = io.openlocal(CSC_TeamsColorsFile)

	if thefile
		local thevalue = thefile:read("*a")
		COM_BufInsertText(player, "supercolorteams "..thevalue.." "..value2)
		thefile:close()
	else
		COM_BufInsertText(player, "supercolorteams 0 "..value2)
	end
end

// Command to set if Red/Blue should be forced for Team gametypes
COM_AddCommand("supercolorteams", function(player, arg, silent)
	if not player
		print("\x85".."ERROR: ".."\x80".."You don't exist!") // Go to a map, please.
		return
	end

	if server != player // Not the owner
	and admin != player // Not an admin.
	and not IsPlayerAdmin(player) // Not an admin. x2
		CSC_PrintIfNotSilent(player, "\x85".."ERROR: ".."\x80".."Only administrators can use this command!", silent) // YOU SHALL NOT PASS!
		return
	end

	if arg
		local og_arg = arg
		arg = string.lower(tostring(arg)) // Case-insensitive

		if arg == "1" // Yes :)
		or arg == "true"
		or arg == "on"
		or arg == "yes"
		or arg == "team"
		or arg == "force"
			for allplayers in players.iterate
				allplayers.csc_teamcolors = "yes"
			end
			CSC_SaveSuperTeams("1")
			CSC_PrintIfNotSilent(player, "Super colors will be forced to ".."\x85".."Red".."\x80".."/".."\x84".."Blue".."\x80".." on Teams gametypes", silent) // Red & Blue colors
		elseif arg == "0" // No >:(
		or arg == "false"
		or arg == "off"
		or arg == "no"
		or arg == "player"
		or arg == "noforce"
			for allplayers in players.iterate
				allplayers.csc_teamcolors = "no"
			end
			CSC_SaveSuperTeams("0")
			CSC_PrintIfNotSilent(player, "Super colors won't be forced on Teams gametypes", silent) // Changed back to default...
		else
			CSC_PrintIfNotSilent(player, "\x85".."ERROR: ".."\x82"..og_arg.."\x80".." isn't a valid option! Printing command guide...", silent) // ...really?
			CSC_PrintIfNotSilent(player, 'supercolorteams <"off"|"on">', silent)
			CSC_PrintIfNotSilent(player, "Toggle if Super colors are changed to adapt to teams", silent)
		end
	else
		CONS_Printf(player, 'supercolorteams <"off"|"on">') // Show a small guide if we don't get any argument
		CONS_Printf(player, "Toggle if Super colors are changed to adapt to teams")
		if player.csc_teamcolors == "yes"
			CONS_Printf(player, "\x82".."Current value: ".."\x83".."ON")
		else
			CONS_Printf(player, "\x82".."Current value: ".."\x85".."OFF")
		end
	end
end)

// If for some reason you prefer to open menus with commands rather than a button combination, csc_setup is for you.
COM_AddCommand("csc_setup", function(player) // arg isn't set because csc_setup doesn't have any arguments
	if not player
		print("\x85".."ERROR: ".."\x80".."You don't exist!") // Go to a map, please.
		return
	end

	if not player.csc_setup_supermenu
		player.csc_setup_supermenu = true

		S_StartSound(nil, sfx_itemup, player)
	else
		player.normalspeed = skins[player.mo.skin].normalspeed

		player.csc_setup_supermenu = false

		player.csc_setup_up = 0
		player.csc_setup_down = 0
		player.csc_setup_left = 0
		player.csc_setup_right = 0

		S_StartSound(nil, sfx_chchng, player)
	end
end)

////////////////////////////////////////////////
// Adding, animating, and applying the colors //
////////////////////////////////////////////////

// Similar to how Sendcolor did, I'll create 32 new colors (1 each player)
for i = 0, 31
	freeslot("SKINCOLOR_CUSTOM_SUPERPLAYER"..i) // Add the colors

	skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+i] = { // Color info.
		name = "Custom Super (Player "..i+(1)..")", // Name ("String")
		ramp = {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31}, // Color ramp ({table}, up to 16 numbers, from 0 to 255)
		invcolor = SKINCOLOR_SILVER, // Opposite color (SKINCOLOR_* constant)
		invshade = 7, // Op. color's shade used for signposts (0-15)
		chatcolor = V_GRAYMAP, // Color used for player's name color on the chat (V_* constant)
		accessible = false // Is it accessible through the Player Setup menu? (boolean)
	}
end

local RampFramesThingy = { // Table that modifies the color ramp
	/* Little tutorial about how to create your own animation
	(I'm not good at explaining stuff and my English just sucks so... yeah...)

	DSICLAIMER: The color ramp animations affect every color!

	You will need the frames for your animation!
	Start off from this structure:

	[x] = y

	'x' should be replaced by the number of the frame, starting from 0.

	[0] = y

	'y' should be a table with 17 values, each one representing a color from the selected
	color's ramp. The color ramp's first color is 0, and the lastest is 15.
	(THE FIRST VALUE IS THERE JUST TO FIX AN ERROR AND WON'T BE USED FOR THE COLOR RAMP!)

	[0] = {0, 2, 4, 6, 8, 10, 12, 14, 1, 3, 5, 7, 9, 11, 13, 15}

	To add more frames, add a comma (,) at the end of the table, and then do the same
	thing again but adding 1 to the frame's number.
	(The lastest frame shouldn't have a comma at the end of its table!)

	[0] = {0, 0, 2, 4, 6, 8, 10, 12, 14, 1, 3, 5, 7, 9, 11, 13, 15},
	[1] = {0, 1, 3, 5, 7, 9, 11, 13, 15, 0, 2, 4, 6, 8, 10, 12, 14}

	In case you want to make a frame look as the original color ramp, instead of doing a
	table, put the "normalcolor" string! (Unless you want to suffer...)

	[0] = {0, 0, 2, 4, 6, 8, 10, 12, 14, 1, 3, 5, 7, 9, 11, 13, 15},
	[1] = {0, 1, 3, 5, 7, 9, 11, 13, 15, 0, 2, 4, 6, 8, 10, 12, 14},
	[2] = "normalcolor"

	And that's it. Have fun creating your own ramp animations if you want to!

    - Dark94
	
	[End of tutorial] */

	/* Commented out, as this exists just for testing purposes.
	[0] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	[1] = {0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
	[2] = {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2},
	[3] = {0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3},
	[4] = {0, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4},
	[5] = {0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5},
	[6] = {0, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6},
	[7] = {0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7},
	[8] = {0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8},
	[9] = {0, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	[10] = {0, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10},
	[11] = {0, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11, 11},
	[12] = {0, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12},
	[13] = {0, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13, 13},
	[14] = {0, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14},
	[15] = {0, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15} */

	[0] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 2, 2, 3, 3}, // 1st frame
	[1] = {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 2, 2, 3, 4, 5, 6, 7}, // 2nd frame
	[2] = {0, 0, 0, 0, 1, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11}, // 3rd frame
	[3] = {0, 0, 1, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13}, // 4th frame
	[4] = "normalcolor" // 5th frame
}

// Load the preferences every time a map loads
addHook("MapLoad", function()
	for player in players.iterate
		CSC_LoadSuperColor(player)
		CSC_LoadSuperTeams(player, true)
	end
end)

addHook("PlayerThink", function(player)
	local plyr = 0
	for i = 0, 31
		if player == players[i]
			plyr = i // Give us our own node ID
		end
	end
	
	for i = 1, #skincolors-1 // Do this to all the skincolors by using [i]
		player.csc_truecolor = player.skincolor

		if player.csc_color == "current"
			player.csc_truecolor = player.skincolor
		elseif player.csc_color == "inv"
			player.csc_truecolor = ColorOpposite(player.skincolor)
		else
			player.csc_truecolor = R_GetColorByName(tostring(player.csc_color))
		end

		if player.csc_teamcolors
		and gametyperules & GTR_TEAMS
		and (player.ctfteam == 1
		or player.ctfteam == 2)
			local teamcolor = SKINCOLOR_GARNET
			if player.ctfteam == 2
				teamcolor = SKINCOLOR_SAPPHIRE
			end

			player.csc_truecolor = teamcolor
		end

		if player.csc_truecolor == i // A player haves the [i] color
			if player.csc_truecolor < SKINCOLOR_CUSTOM_SUPERPLAYER0 // Avoid animating the colors that CSC added, as these animate our Super color!
			or player.csc_truecolor > SKINCOLOR_CUSTOM_SUPERPLAYER0+31
				local anim_timer = leveltime // If we are NOT Super, animate the color with the level's time
				if player.powers[pw_super] // But... if we ARE Super, animate the color with our Super time
					anim_timer = player.powers[pw_super]
				end

				local weirdmath = abs(((anim_timer>>1)%((#RampFramesThingy+1)*2-1))-#RampFramesThingy) // Recreation of the maths that makes animated Super colors

				for j = 1, 16 // Check all the values from our current frame
					local newcolorramp // Store our modified color ramp here
					if RampFramesThingy[weirdmath] == "normalcolor" // Use the original color if we get the "normalcolor" string
						newcolorramp = skincolors[i].ramp[j-1]
					else // Actual numbers, I guess...
						newcolorramp = skincolors[i].ramp[RampFramesThingy[weirdmath][j]]
					end

					// Apply the new information to the player's Super color slot
					skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].ramp[j-1] = newcolorramp
					skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].invcolor = skincolors[i].invcolor
					skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].invshade = skincolors[i].invshade
					skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].chatcolor = skincolors[i].chatcolor
				end
			else
				// Back to default
				skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].ramp = {1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31}
				skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].invcolor = SKINCOLOR_SILVER
				skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].invshade = 7
				skincolors[SKINCOLOR_CUSTOM_SUPERPLAYER0+plyr].chatcolor = V_GRAYMAP
			end
		end
	end

	if player.powers[pw_super]
		if player.csc_realsupertime
			player.csc_realsupertime = $+1
		else
			player.csc_realsupertime = 1
		end
	else
		player.csc_realsupertime = 0
	end

	if player.mo
	and player.mo.valid // We need to exist. Duh.
		if player.mo.health // We are alive
		and ((player.powers[pw_super] // Super?
		and player.powers[pw_super] > 1)
		or (player.csc_realsupertime
		and player.csc_realsupertime > 16))
		and player.csc_color != "none" // Don't change our color if we are using the default Super color
			if not (player.pflags & PF_GODMODE) // Don't affect the Hyper form
			and ((player.hyper // (including XMom's Hyper form)
			and not player.hyper.capable)
			or not player.hyper)
				player.mo.color = CSC_GiveSuperColor(player) // Change our color

				player.csc_appliedcolor = true
			end
		else
			if player.csc_appliedcolor
				player.mo.color = player.skincolor // We're not super anymore? Give us our normal color back!
				if ((player.powers[pw_super]
				and player.powers[pw_super] > 1)
				or (player.csc_realsupertime
				and player.csc_realsupertime > 16))
					player.mo.color = CSC_GiveSuperColor(player) // Oh wait... We STILL are super!
				end
			end

			player.csc_appliedcolor = false
		end
	end
end)

/////////////////
// Misc. stuff //
/////////////////

// Fix afterimages
addHook("MobjThinker", function(mobj)
	if mobj
	and mobj.valid
	and mobj.target
	and mobj.target.valid
	and mobj.target.player
	and mobj.target.player.powers[pw_super]
	and mobj.target.player.powers[pw_super] > 1
		mobj.color = mobj.target.color
	end
end, MT_GHOST)

// Why there are people that don't set opposite colors to their custom skincolors!?
// I guess I'm doing it myself...
addHook("ThinkFrame", function()
	for i = 1, #skincolors-1
		if skincolors[i]
		and not skincolors[i].invcolor
			// Fun Fact: The signpost's default BG color is Green's darkest shade
			skincolors[i].invcolor = SKINCOLOR_GREEN
			skincolors[i].invshade = 15
		end
	end
end)

////////////////////////////////
// Menu Navigation & Behavior //
////////////////////////////////

addHook("PlayerSpawn", function(player)
	player.csc_setup_supermenu = false

	player.csc_setup_idontknowhowshouldicallthisvariable = false

	player.csc_setup_up = 0
	player.csc_setup_down = 0
	player.csc_setup_left = 0
	player.csc_setup_right = 0
end)

addHook("PlayerThink", function(player)
	if player.mo
	and player.mo.valid // We need to exist. Duh.
	and player.mo.health // We are alive
	and not P_PlayerInPain(player)
	and not player.exiting
	and not player.powers[pw_nocontrol]
	and (player.mo.state < S_PLAY_SUPER_TRANS1
	or player.mo.state > S_PLAY_SUPER_TRANS6)
		if player.cmd.forwardmove > 10
			player.csc_setup_up = $+1
		else
			player.csc_setup_up = 0
		end
		if player.cmd.forwardmove < -10
			player.csc_setup_down = $+1
		else
			player.csc_setup_down = 0
		end
		if player.cmd.sidemove < -10
			player.csc_setup_left = $+1
		else
			player.csc_setup_left = 0
		end
		if player.cmd.sidemove > 10
			player.csc_setup_right = $+1
		else
			player.csc_setup_right = 0
		end

		if player.csc_setup_supermenu
			if not player.csc_setup_idontknowhowshouldicallthisvariable
				if player.csc_color
				and player.csc_color != "none"
				and player.csc_color != "current"
				and player.csc_color != "inv"
				and R_GetNameByColor(player.csc_color)
					player.csc_setup_selectedcolor = player.csc_color
				else
					player.csc_setup_selectedcolor = SKINCOLOR_WHITE
				end
			end

			player.csc_setup_idontknowhowshouldicallthisvariable = true

			player.normalspeed = 0

			if player.csc_setup_up == 1
			or player.csc_setup_up >= 20
				if player.csc_color == "current" // Current to Default
					COM_BufInsertText(player, "customsupercolor 0 1")
				elseif player.csc_color == "inv" // Opposite to Current
					COM_BufInsertText(player, "customsupercolor current 1")
				elseif R_GetColorByName(player.csc_color) // Custom to Opposite
					COM_BufInsertText(player, "customsupercolor inv 1")
				else // Default to Custom
					COM_BufInsertText(player, 'customsupercolor "'..tonumber(player.csc_setup_selectedcolor)..'" 1')
				end

				S_StartSound(nil, sfx_s1ba, player)
			end
			if player.csc_setup_down == 1
			or player.csc_setup_down >= 20
				if player.csc_color == "current" // Current to Opposite
					COM_BufInsertText(player, "customsupercolor inv 1")
				elseif player.csc_color == "inv" // Opposite to Custom
					COM_BufInsertText(player, 'customsupercolor "'..tonumber(player.csc_setup_selectedcolor)..'" 1')
				elseif R_GetColorByName(player.csc_color) // Custom to Default
					COM_BufInsertText(player, "customsupercolor 0 1")
				else // Default to Current
					COM_BufInsertText(player, "customsupercolor current 1")
				end

				S_StartSound(nil, sfx_s1ba, player)
			end
			if R_GetColorByName(player.csc_color) // Choosing a custom color
				if player.csc_setup_left == 1 // Left color
				or player.csc_setup_left >= 20
					player.csc_setup_selectedcolor = CSC_GivePreviousColor(player.csc_setup_selectedcolor)

					COM_BufInsertText(player, 'customsupercolor "'..tonumber(player.csc_setup_selectedcolor)..'" 1')
					S_StartSound(nil, sfx_menu1, player)
				end
				if player.csc_setup_right == 1 // Right color
				or player.csc_setup_right >= 20
					player.csc_setup_selectedcolor = CSC_GiveNextColor(player.csc_setup_selectedcolor)

					COM_BufInsertText(player, 'customsupercolor "'..tonumber(player.csc_setup_selectedcolor)..'" 1')
					S_StartSound(nil, sfx_menu1, player)
				end
			end

			if player.cmd.buttons & BT_JUMP
				player.normalspeed = skins[player.mo.skin].normalspeed

				player.csc_setup_supermenu = false

				player.csc_setup_up = 0
				player.csc_setup_down = 0
				player.csc_setup_left = 0
				player.csc_setup_right = 0

				S_StartSound(nil, sfx_chchng, player)
			end
		else
			if player.csc_tf_pressagain_orelse // Press the button again, quick!
				player.csc_tf_pressagain_orelse = $-1
			else // Reset
				player.csc_tf_timespressed = 0
				player.csc_tf_pressagain_orelse = 0
			end
			if player.cmd.buttons & BT_TOSSFLAG // Originally SHIELD + TOSS FLAG, but changed to pressing TOSS FLAG three times quickly in v1.1 due to Shield button's removal in 2.2.15
			and P_IsObjectOnGround(player.mo) // Grounded, please
			and not player.csc_tf_down // Can't open the menu just by holding TF!
				player.csc_tf_timespressed = $+1
				player.csc_tf_pressagain_orelse = 10
			end
			if player.csc_tf_timespressed > 2 // We've presssed TF 3 times in a row!
				player.mo.state = S_PLAY_WAIT

				player.csc_setup_supermenu = true
				S_StartSound(nil, sfx_itemup, player)

				player.csc_tf_timespressed = 0
				player.csc_tf_pressagain_orelse = 0
			end
			if player.cmd.buttons & BT_TOSSFLAG
				player.csc_tf_down = true
			else
				player.csc_tf_down = false
			end

			player.csc_setup_idontknowhowshouldicallthisvariable = false
		end
	else
		if player.csc_setup_supermenu
			player.normalspeed = skins[player.mo.skin].normalspeed

			S_StartSound(nil, sfx_chchng, player)
		end

		player.csc_setup_supermenu = false

		player.csc_setup_idontknowhowshouldicallthisvariable = false

		player.csc_setup_up = 0
		player.csc_setup_down = 0
		player.csc_setup_left = 0
		player.csc_setup_right = 0
	end
end)

////////////////////
// Menu Interface //
////////////////////

local walktime
local newframelimit
local oldplayerskin

hud.add(function(v, player)
	if player.mo
	and player.mo.valid
		if leveltime < 2
		or oldplayerskin != player.mo.skin
		or not player.csc_setup_supermenu
			walktime = 0
			newframelimit = 64
			oldplayerskin = player.mo.skin
		end

		walktime = $+1

		local walkframes = skins[player.mo.skin].sprites[SPR2_WALK].numframes
		local walkanimation = walktime%(walkframes*4)/4
		local superplayerwalk = v.getSprite2Patch(player.mo.skin, SPR2_WALK, true, walkanimation, 1)

		if not superplayerwalk
		or newframelimit != 64
			if newframelimit == 64
				newframelimit = walkanimation
			end
			walkanimation = walktime%(newframelimit*4)/4
			superplayerwalk = v.getSprite2Patch(player.mo.skin, SPR2_WALK, true, walkanimation, 1)

			if not superplayerwalk
				superplayerwalk = v.getSprite2Patch(player.mo.skin, SPR2_WALK, true, A, 1)
			end
		end

		// Menu tip
		if (leveltime < 10*TICRATE-TICRATE/2
		or (leveltime < 10*TICRATE
		and leveltime%2))
		and not CSC_MenuTipDisappeared
			// v.drawFill didn't worked as transparent background sooo.....
			v.draw(160, 30, v.cachePatch("CSC_MENUTIPBG"), V_50TRANS|V_PERPLAYER|V_SNAPTOTOP)
			// text.
			v.drawString(160, 20, "You can change your ".."\x82".."Super".."\x80".." color", V_PERPLAYER|V_SNAPTOTOP, "thin-center")
			v.drawString(160, 30, "by pressing ".."\x87".."TOSSFLAG 3 times".."\x80".." or", V_PERPLAYER|V_SNAPTOTOP, "thin-center")
			v.drawString(160, 40, "by typing ".."\x87".."csc_setup".."\x80".." on the console!", V_PERPLAYER|V_SNAPTOTOP, "thin-center")
		elseif leveltime >= 10*TICRATE
			CSC_MenuTipDisappeared = true // Show the tip once per start-up
		end
		if player.csc_setup_supermenu // Draw the stuff only if we opened the menu
			CSC_MenuTipDisappeared = true // Hide the Menu tip

			v.drawScaled(0, 0, FRACUNIT*10, v.cachePatch("CSC_SETUPBG"), V_50TRANS|V_PERPLAYER|V_SNAPTOTOP|V_SNAPTOLEFT)
			// Show a preview!
			v.drawFill(200, -100, 100, 9999, 253, V_PERPLAYER)
			if skins[player.mo.skin].followitem
			and skins[player.mo.skin].followitem == MT_TAILSOVERLAY
				local thosetails = v.getSprite2Patch(player.mo.skin, SPR2_TAL4, true, walktime%(skins[player.mo.skin].sprites[SPR2_TAL4].numframes*4)/4, 1)
				if player.charability == CA_FLY
				and walktime%(skins[player.mo.skin].sprites[SPR2_TAL4].numframes*4)/4 > 3
					v.drawScaled(250*FRACUNIT, 140*FRACUNIT, skins[player.mo.skin].highresscale*3/2, thosetails, V_PERPLAYER|V_FLIP, v.getColormap(player.mo.skin, CSC_GiveSuperColor(player)))
				else
					v.drawScaled(250*FRACUNIT, 140*FRACUNIT, skins[player.mo.skin].highresscale*3/2, thosetails, V_PERPLAYER, v.getColormap(player.mo.skin, CSC_GiveSuperColor(player)))
				end
			end
			v.drawScaled(250*FRACUNIT, 140*FRACUNIT, skins[player.mo.skin].highresscale*3/2, superplayerwalk, V_PERPLAYER, v.getColormap(player.mo.skin, CSC_GiveSuperColor(player)))
			// SUPER SETUP text
			v.draw(160, 7, v.cachePatch("CSC_SUPERSETUP"), V_PERPLAYER)

			// Default color
			v.drawString(16, 40-10, "Default", V_PERPLAYER)
			if player.csc_color == "none" // Selected?
				v.drawString(16, 40-10, "Default", V_PERPLAYER|V_YELLOWMAP)
				CSC_DrawSquareBorders(v, 16*FRACUNIT, 40*FRACUNIT, 8*FRACUNIT*16, 16*FRACUNIT, V_PERPLAYER)
			end
			v.drawStretched(16*FRACUNIT, 40*FRACUNIT, FRACUNIT*16, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, CSC_GiveColor(player, "super")))

			// Player color
			if player.csc_color == "current" // Selected?
				CSC_DrawSquareBorders(v, 16*FRACUNIT, 75*FRACUNIT, 8*FRACUNIT*16, 16*FRACUNIT, V_PERPLAYER)
			end
			v.drawString(16, 75-10, "Current", V_PERPLAYER|skincolors[player.skincolor].chatcolor)
			v.drawStretched(16*FRACUNIT, 75*FRACUNIT, FRACUNIT*16, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, CSC_GiveColor(player, "current")))

			// Invert color
			if player.csc_color == "inv" // Selected?
				CSC_DrawSquareBorders(v, 16*FRACUNIT, 110*FRACUNIT, 8*FRACUNIT*16, 16*FRACUNIT, V_PERPLAYER)
			end
			v.drawString(16, 110-10, "Opposite", V_PERPLAYER|skincolors[ColorOpposite(player.skincolor)].chatcolor)
			v.drawStretched(16*FRACUNIT, 110*FRACUNIT, FRACUNIT*16, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, CSC_GiveColor(player, "opposite")))

			// Custom color
			local GPC = CSC_GivePreviousColor
			local GNC = CSC_GiveNextColor
			v.drawString(16, 145-10, "Custom", V_PERPLAYER)
			if R_GetColorByName(player.csc_color) // Selected?
				v.drawString(16, 145-10, "Custom", V_PERPLAYER|V_YELLOWMAP)
				v.drawString(76, 145-(10+leveltime%10/5), string.char(27), V_PERPLAYER|V_YELLOWMAP)
				CSC_DrawSquareBorders(v, 16*FRACUNIT, 145*FRACUNIT, 8*FRACUNIT*16, 16*FRACUNIT, V_PERPLAYER)
			end
			v.drawString(16+8*16, 145+18, skincolors[player.csc_setup_selectedcolor].name, V_PERPLAYER|V_ALLOWLOWERCASE|skincolors[player.csc_setup_selectedcolor].chatcolor, "right")
			// dont ask.
			v.drawStretched(16*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GPC(GPC(GPC(GPC(GPC(GPC(player.csc_setup_selectedcolor))))))))
			v.drawStretched((16+8)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GPC(GPC(GPC(GPC(GPC(player.csc_setup_selectedcolor)))))))
			v.drawStretched((16+8*2)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GPC(GPC(GPC(GPC(player.csc_setup_selectedcolor))))))
			v.drawStretched((16+8*3)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GPC(GPC(GPC(player.csc_setup_selectedcolor)))))
			v.drawStretched((16+8*4)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GPC(GPC(player.csc_setup_selectedcolor))))
			v.drawStretched((16+8*5)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GPC(player.csc_setup_selectedcolor)))
			v.drawStretched((16+8*6)*FRACUNIT, 145*FRACUNIT, FRACUNIT*4, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, player.csc_setup_selectedcolor))
			v.drawStretched((16+8*10)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GNC(player.csc_setup_selectedcolor)))
			v.drawStretched((16+8*11)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GNC(GNC(player.csc_setup_selectedcolor))))
			v.drawStretched((16+8*12)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GNC(GNC(GNC(player.csc_setup_selectedcolor)))))
			v.drawStretched((16+8*13)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GNC(GNC(GNC(GNC(player.csc_setup_selectedcolor))))))
			v.drawStretched((16+8*14)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GNC(GNC(GNC(GNC(GNC(player.csc_setup_selectedcolor)))))))
			v.drawStretched((16+8*15)*FRACUNIT, 145*FRACUNIT, FRACUNIT, FRACUNIT, v.cachePatch("CSC_COLORRAMP"), V_PERPLAYER, v.getColormap(TC_DEFAULT, GNC(GNC(GNC(GNC(GNC(GNC(player.csc_setup_selectedcolor))))))))

			// Exit tip
			v.drawString(160, 180, "Press ".."\x82".."[JUMP]".."\x80".." to exit", V_PERPLAYER|V_SNAPTOBOTTOM|V_ALLOWLOWERCASE, "center")

			// Display the current CSC version
			v.drawString(0, 184, "\x87".."C".."\x82".."S".."\x87".."C".."\x80".." version", V_PERPLAYER|V_50TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT, "thin")
			v.drawString(0, 192, CSC_VERSION, V_PERPLAYER|V_50TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_ALLOWLOWERCASE, "thin")
		end
	end
end)

print("\x87".."Custom ".."\x82".."Super ".."\x87".."Color ".."\x80".."has been loaded successfully", "\x83".."Enjoy!")